/* Copyright (c) 2009 Nordic Semiconductor. All Rights Reserved.
 *
 * The information contained herein is confidential property of Nordic
 * Semiconductor ASA.Terms and conditions of usage are described in detail
 * in NORDIC SEMICONDUCTOR STANDARD SOFTWARE LICENSE AGREEMENT.
 *
 * Licensees are granted free, non-transferable use of the information. NO
 * WARRANTY of ANY KIND is provided. This heading must NOT be removed from
 * the file.
 *
 */

/* 
 * This example shows the functionality of the low power comparator (LPCOMP), present
 * in nRF51 series XLR2.
 * To test, connect a voltage source to pin P0.5 (AIN6). 
 * When the voltage goes above the specified reference voltage, pin 10 (LED2) is set.
 * When the voltage goes below the specified reference voltage, pin 10 (LED2) is cleared.
 * When the voltage on AIN6 either goes above or below the specified reference voltage, 
 * pin 11 (LED3) is toggled to show that the LPCOMP interrupt handler is executed.
 *
 * @image html example_board_setup_a.png "Use board setup A for this example."
 */

#include <stdbool.h>
#include "nrf.h"
#include "nrf_gpio.h"

#define EVENT_TRIGGERED_PIN 	10
#define RESULT_PIN						11

/** Configures port 1 as output
 */
static void gpio_config(void)
{
  nrf_gpio_range_cfg_output(8, 15);
  nrf_gpio_port_write(NRF_GPIO_PORT_SELECT_PORT1, 0);
}


/** Configures and enables the LPCOMP
 */
void LPCOMP_init(void)
{	
	/* Enable interrupt on LPCOMP CROSS event */		
	NRF_LPCOMP->INTENSET = LPCOMP_INTENSET_CROSS_Msk;
	NVIC_EnableIRQ(LPCOMP_IRQn);	
	
	/* Configure LPCOMP - set input source to AVDD*4/8 */
	NRF_LPCOMP->REFSEL |= (LPCOMP_REFSEL_REFSEL_SupplyFourEighthsPrescaling << LPCOMP_REFSEL_REFSEL_Pos);
	/* Configure LPCOMP - set reference input source to AIN pin 6, i.e. P0.5 */
	NRF_LPCOMP->PSEL |= (LPCOMP_PSEL_PSEL_AnalogInput6 << LPCOMP_PSEL_PSEL_Pos);
	
	/* Enable and start the low power comparator */
	NRF_LPCOMP->ENABLE = LPCOMP_ENABLE_ENABLE_Enabled;	
	NRF_LPCOMP->TASKS_START = 1;
}



/* Interrupt handler for LPCOMP */
void LPCOMP_COMP_IRQHandler(void)
{
	// Clear event
	NRF_LPCOMP->EVENTS_CROSS = 0;
	
	// Sample the LPCOMP stores its state in the RESULT register. 
	// RESULT==0 means lower than reference voltage, 
	// RESULT==1 means higher than reference voltage
	NRF_LPCOMP->TASKS_SAMPLE = 1;
	nrf_gpio_pin_write(RESULT_PIN, NRF_LPCOMP->RESULT);
	
	// Toggle pin to indicate triggering of event
	nrf_gpio_pin_toggle(EVENT_TRIGGERED_PIN);
}


/**
 * main function
 */
int main(void)
{
  gpio_config();
	LPCOMP_init();
	
  while (true)
  {	
		//Put CPU to sleep while waiting for interrupt to save power
		__WFI();
  }
}

/**
 *@}
 **/
